r'''
NOTES:
    System functions
TASK:
WARNINGS:
'''

__update__ = '2023.09.21'
__author__ = 'PABLO GONZALEZ PILA <pablogonzalezpila@gmail.com>'

''' SYSTEM LIBRARIES '''
import os, datetime, platform, time
import hashlib
import urllib.request # INTERNET CONNECTION
import pyperclip as clipboard # Portapapeles
from inspect import getmembers, isfunction, isclass # OBJECTS CHECK
from dataclasses import dataclass, fields, MISSING
# from importlib import import_module


''' FUNCTIONS
--------------------------------------------------------
'''

## OPERATING SYSTEM

def GET_FIRM() -> str:
    '''
    Returns string with OS Login Id and Date in format ' yyyy-mm-dd / hh:mm '
    '''
    FIRM = "{} [{}]".format(os.getlogin(), datetime.datetime.now().strftime("%Y-%m-%d / %H:%M"))
    return FIRM

def OS_GET_SYSTEM() -> str:
    '''
    Returns the system OS name 
    '''
    os = platform.system()
    return os

def OS_GET_LOGIN() -> str:
    '''
    Get User SO Log-In Id in str format
    '''
    value = os.getlogin()
    return value

def COPY2CLIPBOARD(TEXT) -> None:
    '''
    Add selected data to clipboard
    '''
    clipboard.copy(TEXT)

def INTERNET_CONNECTION_CHECK(URL=r'https://cloud.seatable.cn/') -> bool:
    '''
    Returns bool value if run the selected URL connection
    Info: Some generics URL not run as well
    '''
    # https://cloud.seatable.cn/
    # https://www.rohde-schwarz.com/

    try:
        urllib.request.urlopen(
            URL,
            timeout=5
            )
        return True
    except:
        return False


## PATH FUNCTIONS

def PATH_OPEN(PATH) -> None:
    ''' 
    Run from OS selected PATH
    '''
    os.startfile(PATH)

def PATH_FOLDER_NEW(PATH=str, FOLDER_NAME=str) -> None:
    '''
    Create a new folder in selected path
    '''
    NEW_FOLDER = os.path.join(PATH,FOLDER_NAME)
    os.mkdir(NEW_FOLDER)

def PATH_GET_DESKTOP() -> str:
    '''
    Returns Desktop Path in str format
    
    INCOMPLETE: 
    - Hay que añadir mas sistemas operativos
    '''
    SO = OS_GET_SYSTEM()
    if SO == 'Windows': 
        PATH = "C:/Users/" + os.getlogin() + "/Desktop/"
    return PATH

def PATH_EXIST_CHECK(PATH=str) -> bool:
    '''
    Check if selected path or file exist 
    '''
    check = os.path.exists(PATH)
    return check



## DATE FUNCTIONS

def DATE_GET_TODAY():
    '''
    Returns Date in str format like ' yyyy-mm-dd '
    '''
    DATE = datetime.datetime.now().strftime('%Y-%m-%d')
    return DATE

def DATE_GET_NOW(format=r'%Y-%m-%d / %H:%M'):
    '''
    Returns Date Time in str (default) format like 'yyyy-mm-dd / hh:mm'
    '''
    DATE = str(datetime.datetime.now().strftime(format))
    return DATE

def DATE_ISO(DATE) -> str:
    '''
    Convert date to ISO Format 'yyyy-mm-dd'
    '''
    iso_date = "{}-{}-{:02d}".format(DATE.year(), DATE.month(), DATE.day())
    return iso_date



## LICENSE FUNCTIONS

def ENCODE_STR(STR: str) -> str:
    code = hashlib.sha256(str(STR).encode('utf-8')).hexdigest()
    return code

def LICENSE_TXT(file, appName: str, loginId: str, limitOpens: int, limitDate: int):
    '''
    Make a LICENSE FILE with the next atributes:
    - file (str) / Address and name of the file
    - appName (str) / Name of the Application
    - loginId (str) / Id of the OS User
    - limitOpens (int) / Maximun Opens
    - limitDate (int) / Maximun Date example: 20230601
    
    INCOMPLETE:
        - Hay que añadir la fecha de creación
    '''
    ## INIT FILE
    with open(file, 'w') as f:
        f.write('')
    ## ENCODE
    app = f"@{appName}"
    app = hashlib.sha256(app.encode('utf-8')).hexdigest()
    login = f"@{loginId}"
    login = hashlib.sha256(login.encode('utf-8')).hexdigest()
    opens = f"@{limitOpens}"
    opens = hashlib.sha256(opens.encode('utf-8')).hexdigest()
    date = f"@{limitDate}"
    date = hashlib.sha256(date.encode('utf-8')).hexdigest()
    ## CREATION TIME FILE
    createTime = os.path.getctime(file)
    createTime = time.ctime(createTime)
    createTime = f"@{createTime}"
    createTime = hashlib.sha256(createTime.encode('utf-8')).hexdigest()
    ## MODIFICATION TIME FILE
    modTime = os.path.getmtime(file)
    modTime = time.ctime(modTime)
    modTime = f"@{modTime}"
    modTime = hashlib.sha256(modTime.encode('utf-8')).hexdigest()
    ## EDIT FILE
    txtList = [app, login, opens, date, createTime, modTime]
    with open(file, 'w') as f:
        f.write('\n'.join(txtList))

def LICENSE_TXT_DATA(file) -> dict:
    '''
    Get data in the file

    INCOMPLETE:
    - Esta funcion se usa una vez dentro de la APP poder obtener los valores de la licencia
    - Hay que integrar esta funcion dentro de LICENSE_TXT_CHECK
    '''
    data = {
        'login': "FAIL",
        'opens': 0,
        'date': 0,
        }
    
    ## OPEN FILE
    if os.path.exists(file):
        with open(file, 'r') as f:
            txt = f.readlines()
    else:
        return data
    ## LOGIN
    login = f"@{OS_GET_LOGIN()}"
    login = hashlib.sha256(login.encode('utf-8')).hexdigest()
    if login == txt[1].replace(chr(10), ""): 
        data['login'] = "PASS"
    ## OPENS
    opens = 0
    for i in range(0, 2000):
        hash = f"@{i}"
        hash = hashlib.sha256(hash.encode('utf-8')).hexdigest()
        if hash == txt[2].replace(chr(10), ""):
            opens = i
            # print("limitOpens: PASS", opens)
            data['opens'] = opens
            break
        if i == 1999:
            # print("ERROR / limitOpens")
            data['opens'] = 0
            # return False
    ## DATE
    date = 0
    for i in range(20230101, 20250101):
        hash = f"@{i}"
        hash = hashlib.sha256(hash.encode('utf-8')).hexdigest()
        if hash == txt[3].replace(chr(10), ""):
            date = i
            # print("limitDate: PASS", date)
            data['date'] = date
            break
    return data

def LICENSE_TXT_CHECK(file, appName: str) -> bool:
    '''
    appName, loginId, limitOpens, limitDate, modTime

    INCOMPLETE:
        - Si no encuentra el fichero devuelve False
    '''

    ## OPEN FILE
    if os.path.exists(file) == False:
        return False
    
    with open(file, 'r') as f:
        txt = f.readlines()

    ## CHECK
    if len(txt) == 6:
        # 
        app = f"@{appName}"
        app = hashlib.sha256(app.encode('utf-8')).hexdigest()
        if app == txt[0].replace(chr(10), ""): print("appName: PASS")
        else:
            # print("ERROR / appName")
            return False
        # 
        login = f"@{os.getlogin()}"
        login = hashlib.sha256(login.encode('utf-8')).hexdigest()
        if login == txt[1].replace(chr(10), ""): print("loginId: PASS")
        else:
            # print("ERROR / loginId")
            return False
        # 
        opens = None
        for i in range(0, 2000):
            hash = f"@{i}"
            hash = hashlib.sha256(hash.encode('utf-8')).hexdigest()
            if hash == txt[2].replace(chr(10), ""):
                opens = i
                print("limitOpens: PASS", opens)
                break
            if i == 1999:
                # print("ERROR / limitOpens")
                return False
        # 
        for i in range(20230101, 20250101):
            hash = f"@{i}"
            hash = hashlib.sha256(hash.encode('utf-8')).hexdigest()
            if hash == txt[3].replace(chr(10), ""):
                date = i
                # print("limitDate: PASS", date)
                break
            if i == 20250100:
                # print("ERROR / limitDate")
                return False
        # 
        createTime = os.path.getctime(file)
        createTime = time.ctime(createTime)
        createTime = f"@{createTime}"
        createTime = hashlib.sha256(createTime.encode('utf-8')).hexdigest()
        if createTime == txt[4].replace(chr(10), ""): print("createTime: PASS")
        else:
            # print("ERROR / createTime")
            return False
        # 
        modTime = os.path.getmtime(file)
        modTime = time.ctime(modTime)
        modTime = f"@{modTime}"
        modTime = hashlib.sha256(modTime.encode('utf-8')).hexdigest()
        if modTime == txt[5].replace(chr(10), ""): print("modTime: PASS")
        else:
            # print("ERROR / modTime")
            return False
        
        ## EDIT FILE
        opens -= 1
        opens = f"@{opens}"
        opens = hashlib.sha256(opens.encode('utf-8')).hexdigest()
        # 
        with open(file, 'w') as f:
            f.write('')
        now = os.path.getmtime(file)
        now = time.ctime(now)
        now = f"@{now}"
        now = hashlib.sha256(now.encode('utf-8')).hexdigest()
        with open(file, 'w') as f:
            f.write('\n'.join(
                [txt[0].replace(chr(10), ""), 
                txt[1].replace(chr(10), ""), 
                opens, 
                txt[3].replace(chr(10), ""), 
                createTime, 
                now
                ]))
        # 
        return True
    # 
    else: 
        return False



## IMAGES FUNCTIONS

def IMG_PIXEL2CM(PIXELS=float) -> float:
    '''
    Convert from pixels value to centimeters value
    '''
    value = PIXELS * 0.0264583333
    return value

def IMG_CM2PIXEL(CM=float) -> float:
    '''
    Convert from centimeters value to pixels value 
    '''
    value = CM * 37.795275591
    return value



## MISCELLANEOUS FUNCTIONS

def OBJECT_CHECK(OBJECT, objectType: str = 'function', onlyNames: bool = False) -> list:
    '''
    - OBJECT: Library from importlib
    - objectType: 'function' (for function objects) / 'class' (for class objects)
    - onlyNames:
        - True (Return a List with al the selected objects in the library)
        - False (Return a List with a tuple with object name and the object)
    
    INCOMPLETE:
    - Cuando el objeto es creado desde importlib no se pasa path
    - No he comprobado si puedes pasar un path, directamente un import, etc.
    '''
    if objectType != 'function' and objectType != 'class':
        print("OBJECT_CHECK ERROR / objectType not supported")
        return []
    if objectType == 'function':
        LIST = getmembers(OBJECT, isfunction)
    if objectType == 'class':
        LIST = getmembers(OBJECT, isclass)
    if onlyNames:
        LIST = [func[0] for func in LIST]
    return LIST

def INT_TWODIGITS(INT=int) -> str:
    '''
    Convert Integer to tow digit string
    1 --> 01
    '''
    ORDR = '{:02d}'.format(INT)
    return ORDR

def dataClass_to_dict(DATACLASS: dataclass) -> dict:
    '''
    Get a dictionary from a dataclass
    '''
    DICT = dict()
    for field in fields(DATACLASS):
        FIELD = field.name
        TYPE = field.type
        DEFAULT = field.default
        if DEFAULT == MISSING:
            DICT[FIELD] = TYPE
        else:
            DICT[FIELD] = DEFAULT
    return DICT



''' TEST
--------------------------------------------------------
'''

def OPEN_DIRECTORY(PATH=str) -> None:
    '''
    INCOMPLETE:
    - Se usa la funcion 'system' para lanzar un comando en consola
    - Para lo que se pensó esta funcióin es mejor usar PATH_OPEN()
    '''
    path = os.path.realpath(PATH)
    os.system(f'start {path}')

pass




r''' 
NOTES:
	Creación de documentos PDF sencillos en A4 (de momento)
	Texto enriquecido
TASK:
	- Añadir la posiblidad de usar otro tipo de tamaños de papel
	- Añadir control de linea a todos las las funciones
	- Añadir cuenta automática de páginas
WARNINGS:
	- Borrar funciones de Test
'''

__update__ = '2023.09.19'
__author__ = 'PABLO GONZALEZ PILA <pablogonzalezpila@gmail.com>'

''' SYSTEM LIBRARIES '''
from dataclasses import dataclass

''' CUSTOM MAIN LIBRARIES '''


''' 
REPORTLAB FUNCTIONS
-------------------------------------------------------- 
'''

from reportlab.pdfgen import canvas
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase import pdfmetrics
from reportlab.lib import pagesizes
from reportlab.lib import colors
from reportlab.lib.utils import ImageReader

@dataclass
class fontTypes(str):
	'''
	DataClass to define Font Types in text format
	'''
	normal: str = "Normal"
	bold: str = "Bold"
	italic: str = "Italic"
	bold_italic: str = "Bold_Italic"

@dataclass
class richText():
	'''
	DataClass for defining the content and formatting of a text string
	'''
	value: str = ""
	font: fontTypes = fontTypes.normal
	size: float = 8
	color: colors = colors.black
		
class PDFREPORT:
	'''
	Un A-4 a 72 ppp    595 x 842
	Default Font: Arial
	'''
	def __init__(self, filePath, docTitle: str, defaultFont=fontTypes.normal, defaultSize=8, defaulColor=colors.black):
		self.filePath = filePath
		self.PDF = canvas.Canvas(self.filePath + ".pdf")

		## DOCUMENT NAME
		if docTitle:
			self.PDF.setTitle(docTitle)
		else:
			self.PDF.setTitle("REPORT")
		
		## FONTS
		# pdfmetrics.registerFont(TTFont('Arial', 'Arial.ttf'))
		# pdfmetrics.registerFont(TTFont('Times_new-BoldItalic', 'timesbi.ttf', subfontIndex=3))
		pdfmetrics.registerFont(TTFont('Normal', 'arial.ttf'))
		pdfmetrics.registerFont(TTFont('Bold', 'arialbd.ttf'))
		pdfmetrics.registerFont(TTFont('Italic', 'ariali.ttf'))
		pdfmetrics.registerFont(TTFont('Bold_Italic', 'arialbi.ttf'))
		self.defaultFont = defaultFont
		self.defaultSize = defaultSize
		self.defaulColor = defaulColor
		# self.SET_FONT(fontTypes.normal, 8)
		self.SET_DEFAULT()
		
		## PAGE SIZE / MARGINS
		pageSize = pagesizes.landscape(pagesizes.A4)
		self.pageHeight = pageSize[0] # 841.88
		self.pageWidth = pageSize[1] # 595.27
		self.marginLeft = 45
		self.marginRight = self.pageWidth - self.marginLeft
		self.marginTop = 10
		self.marginBottom = self.pageHeight - 10
		self.row = 820 - self.marginTop
		self.spacing = 5

		## Get objects
		# lineWidth = self.PDF._lineWidth
		# print(lineWidth)
		# fillColor = self.PDF._fillColorObj
		# print(fillColor)

		## SAVING THE .PDF FILE
		# self.PDF.save()
		pass

	def WR_LINE(self, x=int, y=int, TXT=str):
		'''
		Writte a line text with the default font config
		'''
		xpos = x
		ypos = y
		words = TXT.split(chr(32))
		for word in words:
			wordWidth = pdfmetrics.stringWidth(word + " ", self.PDF._fontname, self.PDF._fontsize)
			if xpos + wordWidth > self.marginRight:
				xpos = self.marginLeft
				ypos -= (self.defaultSize + self.spacing)
			self.PDF.drawString(xpos,ypos,word)
			xpos += wordWidth + 1
		self.row = ypos - (self.defaultSize + self.spacing)
	
	def WR_LINE_CENTERED(self, x=int, y=int, TXT=str):
		self.PDF.drawCentredString(x,y,TXT)
		self.row -= (self.PDF._fontsize + self.spacing)
	
	def WR_MULTILINE(self):
		'''
		INCOMPLETE
		'''
		# text = self.PDF.beginText(40, 680)
		# text.setFont("Courier", 18)
		# text.setFillColor(colors.red)
		# for line in textLines:
		# 	text.textLine(line)
		# self.PDF.drawText(text)
		pass

	def WR_HEADER(self, x=int, y=int, TXT=str, filling=None, fontType=fontTypes.bold, size=15):
		'''
		hay que controlar el ancho del HEADER
		'''
		ypos = y - size
		## FILLING
		self.PDF.setFillColorRGB(100, 100, 100)
		# textWidth = pdfmetrics.stringWidth(TXT + "  ", fontType, size)
		self.PDF.setLineWidth(0)
		self.PDF.setLineCap(0)
		# self.PDF.rect(report.marginLeft,y,textWidth,size, fill=1)
		# self.PDF.rect(self.marginLeft,ypos,self.marginRight-30,size, fill=1)
		## TEXT
		self.PDF.setFillColor(colors.black)
		self.PDF.setFont(fontType, size)
		self.PDF.drawString(x,ypos,TXT)
		self.row = ypos
		self.WR_SPACING(2)
		## SET DEFAULT
		self.SET_DEFAULT()

	def WR_RICHTEXT(self, x=int, y=int, TXT_LIST=list):
		'''
		Escribe una linea tanto en formato str como con richText respetando el ancho de la página
		'''
		xpos = x
		ypos = y
		for item in TXT_LIST:
			# Set Text
			if type(item) == str:
				txt = item
			if type(item) == int or type(item) == float:
				txt = str(item)
			if type(item) == richText:
				self.SET_FONT(item.font, item.size)
				self.SET_COLOR(item.color)
				txt = str(item.value)
			# 
			words = txt.split(chr(32))
			for word in words:
				wordWidth = pdfmetrics.stringWidth(word + " ", self.PDF._fontname, self.PDF._fontsize)
				if xpos + wordWidth > self.marginRight:
					xpos = self.marginLeft
					ypos -= (self.defaultSize + self.spacing)
				self.PDF.drawString(xpos,ypos,word)
				xpos += wordWidth + 1
			# Return to default font config
			self.SET_DEFAULT()
		# 
		self.row = ypos - (self.defaultSize + self.spacing)

	def WR_HLINE(self, y=int, lineWidth: int=1.5):
		'''
		Draw a horizontal line
		'''
		self.PDF.setLineWidth(lineWidth) # Line Width
		self.PDF.line(self.marginLeft, y, self.marginRight, y)
		self.WR_SPACING(2)

	def WR_SPACING(self, lines=1):
		self.row -= (self.spacing * 2 * lines)

	def IMAGE_INSERT(self, imageFile, x=int, y=int, ):
		'''
		INCOMPLETE
		Hay que añadir la relacion de aspecto
		'''
		## drawing a image at the specified (x.y) position
		image = ImageReader(imageFile)
		image = imageFile
		self.PDF.drawInlineImage(image, x, y)
		pass

	def FONT_REGISTRER(self, fontName=str, fontTTF="Arial.ttf"):
		pdfmetrics.registerFont(TTFont(fontName, fontTTF))
	
	def SET_DEFAULT(self):
		'''
		Return to default font config
		'''
		self.SET_FONT(self.defaultFont, self.defaultSize)
		self.PDF.setFillColor(self.defaulColor)

	def SET_FONT(self, fontType: fontTypes, fontSize: int):
		'''
		Set the font Type and font Size
		'''
		self.PDF.setFont(fontType, fontSize)

	def SET_COLOR(self, color=colors, rgb=tuple):
		'''
		INCOMPLETE:
			- Hay que crear un dataclass con el color
		'''
		# if color == "black": self.PDF.setFillColorRGB(0, 0, 0)
		# if color == "blue": self.PDF.setFillColorRGB(0, 0, 255)
		# if color == "red": self.PDF.setFillColorRGB(255, 0, 0)
		# if color == "green": self.PDF.setFillColorRGB(0, 255, 0)
		# if color == "grey": self.PDF.setFillColorRGB(160, 160, 160)
		self.PDF.setFillColor(color)
		color_rgb = self.PDF._fillColorObj
		# print("SET_COLOR:", color_rgb)
		pass

''' TEST
-------------------------------------------------------- 
'''


r'''
NOTES:
    -
TASK:
    - Implementar la comunicación por RS-232
WARNINGS
    - al usar "sleep" crashea desde una app con PyQt, estar atento (19.09.2023)
'''

__update__ = '2023.09.20'
__author__ = 'PABLO GONZALEZ PILA <pablogonzalezpila@gmail.com>'

''' SYSTEM LIBRARIES '''
from dataclasses import dataclass
# from time import sleep

''' VISA INSTRUMENT
--------------------------------------------------------  '''

class VISA_INSTRUMENT:
    '''
    visa_resource: GPIB0::17::INSTR" / USB0::0x0AAD::0x014E::101060::INSTR / PXI1Slot2\n
    timeout (seconds) = 10 seconds default
    '''
    def __init__(self, visa_resource: str, timeout: int = 10):
        import pyvisa
        RM = pyvisa.ResourceManager()
        RM.list_resources()
        self.DEVICE = RM.open_resource(visa_resource)
        self.DEVICE.timeout = timeout * 1000 # miliseconds
    
    def CLOSE(self):
        self.DEVICE.close()
    
    def WR(self, SENTENCE: str):
        '''
        Write
        '''
        self.DEVICE.write(SENTENCE)
    
    def RD(self, SENTENCE: str, FLOAT: bool = False) -> str | float:
        '''
        Query
        '''
        VALUE = self.DEVICE.query(SENTENCE)
        if FLOAT:
            VALUE = float(VALUE)
        return VALUE

    def READ(self, FLOAT: bool = False) -> str | float:
        '''
        Read
        '''
        VALUE = self.DEVICE.read()
        if FLOAT:
            VALUE = float(VALUE)
        return VALUE
    
    def OPC(self) -> None:
        self.DEVICE.write("*CLS")
        self.DEVICE.write("ESE 1")
        self.DEVICE.write("SRE 32")
        self.DEVICE.write("*OPC?")
        self.DEVICE.read()


''' RS232 INSTRUMENT
--------------------------------------------------------  '''

class RS232_INSTRUMENT:
    '''
    INCOMPLETE:
        - Not tested yet 
    '''
    def __init__(self, port: str = "COM1", baudrate: int = 9600, timeout: float = 1) -> None:
        import serial
        self.DEVICE = serial.Serial(port, baudrate, timeout)

    def CLOSE(self):
        self.DEVICE.close()

    def WR(self, SENTENCE: str):
        '''
        Write
        '''
        bytes_value = SENTENCE.encode('utf-8')
        self.DEVICE.write(bytes_value)

    def RD(self, SENTENCE: str, FLOAT: bool = False) -> str | float:
        '''
        Query
        '''
        bytes_value = SENTENCE.encode('utf-8')
        self.DEVICE.write(bytes_value)
        VALUE = self.DEVICE.readline().decode('utf-8')
        if FLOAT:
            VALUE = float(VALUE)
        return VALUE


''' PXI INSTRUMENT
--------------------------------------------------------  '''

''' NI MODULES
NI-DCPower (Python module: nidcpower)
NI-Digital Pattern Driver (Python module: nidigital)
NI-DMM (Python module: nidmm)
NI-FGEN (Python module: nifgen)
NI-ModInst (Python module: nimodinst)
NI-SCOPE (Python module: niscope)
NI Switch Executive (Python module: nise)
NI-SWITCH (Python module: niswitch)
NI-TClk (Python module: nitclk)
'''

class PXI_DMM:
    '''
    INCOMPLETE:
    - En caso de error hay que devolver un false
    - Hay que cerrar session con self.session.close
    - Al usar range=-1 lo configuramos en AUTO
    - Hay que buscar las opciones de NULL
    - Añadir espera o indicación al terminar el self test y el self cal
    '''
    def __init__(self, resource: str = ""):
        import nidmm
        self.session = nidmm.Session(resource)
        self.NMB_FUNCTIONS: tuple = (
            self.MEAS,
            self.CONFIG_VDC,
            self.CONFIG_VAC,
            self.CONFIG_RES_2W,
            self.CONFIG_RES_4W,
            self.CONFIG_IDC,
            self.CONFIG_IAC,
            self.CONFIG_FREQ,
            self.CONFIG_TEMP
        )
    
    def CLOSE(self) -> None:
        self.session.close()

    def DEVICE_INFO(self) -> str:
        '''
        Get info (Manufacturer, Model, Serial Id) about device
        '''
        MANUFACTURER = self.session.instrument_manufacturer
        MODEL = self.session.instrument_model
        SERIAL_NUMBER = self.session.serial_number
        idn = f"{MANUFACTURER},{MODEL},{SERIAL_NUMBER}"
        return idn

    def SELF_TEST(self) -> None:
        self.session.self_test()

    def SELF_CAL(self) -> None:
        self.session.self_cal()

    def MEAS_INFO(self) -> str:
        '''
        Get info (Measure Function, Range, Digits) about config
        '''
        FUNCTION = self.session.function
        RANGE = self.session.range
        DIGITS = self.session.resolution_digits
        config = f"{FUNCTION}, {RANGE}, {DIGITS}"
        return config

    def MEAS(self, *args) -> float:
        measure = self.session.read()
        try:
            measure = float(measure)
            return measure
        except:
            print("MEAS ERROR / Float value")
            return 0.0  

    def CONFIG_VDC(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.DC_VOLTS,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )
    
    def CONFIG_VAC(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.AC_VOLTS,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )
    
    def CONFIG_RES_2W(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.TWO_WIRE_RES,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )

    def CONFIG_RES_4W(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.FOUR_WIRE_RES,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )

    def CONFIG_IDC(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.DC_CURRENT,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )

    def CONFIG_IAC(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.AC_CURRENT,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )

    def CONFIG_FREQ(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.FREQ,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )

    def CONFIG_TEMP(self, *args) -> None:
        '''
        '''
        ## RANGE
        if len(args) > 0 and args[0] and args[0] != "": 
            range_value = float(args[0])
        else: 
            range_value: float = -1
        ## DIGITS
        if len(args) > 1 and args[1] and args[1] != "": 
            digits_value = float(args[0])
        else: 
            digits_value: float = 6.5
        ## CONFIG
        self.session.configure_measurement_digits(
                measurement_function=nidmm.Function.TEMPERATURE,
                range = float(range_value),
                resolution_digits= float(digits_value)
                )

class PXI_DCPOWER:
    '''
    INCOMPLETE
    '''
    def __init__(self, resource: str = ""):
        import nidcpower
        self.session = nidcpower.Session(resource)
    
    def CLOSE(self) -> None:
        self.session.close()

    def DEVICE_INFO(self) -> str:
        '''
        Get info (Manufacturer, Model, Serial Id) about device
        '''
        MANUFACTURER = self.session.instrument_manufacturer
        MODEL = self.session.instrument_model
        SERIAL_NUMBER = self.session.serial_number
        idn = f"{MANUFACTURER},{MODEL},{SERIAL_NUMBER}"
        return idn

class NI_VBENCH:
    '''
    INCOMPLETE
    '''
    def __init__(self, resource: str):
        from pyvirtualbench import PyVirtualBench, PyVirtualBenchException, Waveform
        self.virtualbench = PyVirtualBench(resource)
        self.fgen = self.virtualbench.acquire_function_generator()

    def DEVICE_INFO(self) -> str:
        '''
        INCOMPLETE
        '''
        MANUFACTURER = "NATIONAL INSTRUMENTS"
        MODEL = self.virtualbench.device_name
        SERIAL_NUMBER = ""
        idn = f"{MANUFACTURER},{MODEL},{SERIAL_NUMBER}"
        return idn

    # def FGEN_ONOFF(self):
    #     self.fgen.run()
    #     self.fgen.

    def FGEN_SETUP(self,
                waveform_function: str,
                frequency: float, # Hz
                amplitude: float, # Vpp
                dc_offset: float, # V
                duty_cycle: float, # %
                ) -> None:
        '''
        waveform_function: DC / SINE / SQUARE / TRIANGLE
        frequency (Hz) / amplitude (Vpp) / dc_offset (V) / duty_cycle (%)
        '''
        if waveform_function == "DC":
            waveForm = Waveform.DC
        if waveform_function == "SINE":
            waveForm = Waveform.SINE
        if waveform_function == "SQUARE":
            waveForm = Waveform.SQUARE
        if waveform_function == "TRIANGLE":
            waveForm = Waveform.TRIANGLE
        self.fgen.configure_standard_waveform(waveForm, amplitude, dc_offset, frequency, duty_cycle)


''' SPECIAL INSTRUMENT
--------------------------------------------------------  '''

# SPECIAL_INSTRUMENTS: tuple = (FLKE_5XXX, PXI_DMM)

@dataclass
class SPECIAL_INSTRUMENTS:
    FLKE_5XXX = FLKE_5XXX
    RS_SM = RS_SM
    PXI_DMM = PXI_DMM

class FLKE_5XXX(VISA_INSTRUMENT):
    '''
    Especial functions for the device:
        - TYPE: CALIBRATOR
        - MANUFACTURER: FLUKE
        - MODEL: 5xxx
    '''
    def __init__(self, visa_resource=str, timeout: int = 10):
        super().__init__(visa_resource, timeout)
        self.WR("*CLS")
        IDN = self.RD("*IDN?; *WAI")
        MODEL = IDN.split(chr(44))[1]
        self.MODEL = None
        if MODEL[0:2] == "57":
            self.MODEL = "5700"
        if MODEL[0:2] == "55":
            self.MODEL = "5500"
        
        ## NMBv3 FUNCTIONS
        self.NMB_FUNCTIONS: tuple = (
            self.OPER,
            self.STBY,
            self.OUT_VPP,
            self.FOUR_WIRES
        )
    
    def DEVICE_INFO(self) -> str:
        self.WR("*CLS")
        IDN = self.RD("*IDN?; *WAI")
        IDNL = IDN.split(chr(44))
        MANUFACTURER = IDNL[0]
        MODEL = IDNL[1]
        SERIAL_NUMBER = IDNL[2]
        idn = f"{MANUFACTURER},{MODEL},{SERIAL_NUMBER}"
        return idn
    
    def OPER(self, VALUE: str, UNIT: str) -> None:
        '''
        '''
        self.WR("*CLS")
        self.WR("ESE 1")
        self.WR("SRE 32")
        self.WR("OPER")
        self.OPC()

    def STBY(self, *args) -> None:
        '''
        '''
        self.WR("*CLS")
        self.WR("ESE 1")
        self.WR("SRE 32")
        self.WR("OUT 0 HZ")
        self.WR("OUT 0 V")
        self.WR("STBY")
        self.OPC()

    def OUT_VPP(self, *args):
        pass

    def FOUR_WIRES(self, *args) -> None:
        pass

class RS_SM(VISA_INSTRUMENT):
    '''
    Especial functions for the device:
        - TYPE: SIGNAL GENERATOR
        - MANUFACTURER: ROHDE & SCHWARZ
        - MODEL: SMx
    '''
    def __init__(self, visa_resource: str, timeout: int = 10):
        super().__init__(visa_resource, timeout)
        
        ## NMBv3 FUNCTIONS
        self.NMB_FUNCTIONS: tuple = (
            self.STBY,
            self.NRP_MEAS,
            self.NRP_ALC
        )
    
    def STBY(self, *args):
        self.WR("*CLS")
        self.WR("ESE 1")
        self.WR("SRE 32")
        self.WR("SOUR:POW -100 DBM")
        self.WR("OUTP:STAT OFF")
        self.OPC()
    
    def NRP_MEAS(self, *args):
        '''
        '''
        pass

    def NRP_ALC(self, *args):
        '''
        Set Power Level to Target Level in NRP
        ** Make sure the generator has a NRP connected and is identified as 1
        INCOMPLETE:
        '''
        # Chekear la sonda
        # Revisar el modelo
        # En caso de fallo llamar STBY
        pass


''' TEST
--------------------------------------------------------  '''


__version__ = '2023.09.21'
__author__ = 'PABLO PILA'
__author_email__ = "pablogonzalezpila@gmail.com"

''' 
NOTES:
TASK:
WARNINGS:
'''

from setuptools import setup, find_packages

setup(
    name = "batteryTools",
    # packages = find_packages(), # con find_pachages no conseguir hacerlo funcionar
    packages=["batteryTools"],
    include_package_data=True, # muy importante para que se incluyan archivos sin extension .py
    version = __version__,
    author = __author__,
    author_email = __author_email__,
    url = "https://github.com/PaulFilms/battery-tools.git",
    long_description = "README.md",
    license = "www.unlicense.org",
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        ]
)



r'''
NOTES:
    Conjunto de librerias para uso de excel
TASK:
    Ver actualizaciones de openpyxl para RichText
    Usar el en los metodos self.ROW
    Añadir la funcion de trabajar con letras y enteros:
        ejm: [ COL = xls.utils.get_column_letter(COLUMN) ]
    Añadir libreria xlsxwriter para uso de hojas EXCEL ya creadas
    ** No me termina de gustar es usar save en los metodos
WARNINGS:
    < xlsxwriter > Instalado para usar texto enriquecido (En pruebas)
'''

__update__ = '2023.09.01'
__author__ = 'PABLO GONZALEZ PILA <pablogonzalezpila@gmail.com>'

''' SYSTEM LIBRARIES '''
import os
import pandas as pd

''' CUSTOM MAIN LIBRARIES '''

''' 
OPENPYXL REPORT
--------------------------------------------------------
'''

import openpyxl as xls
from openpyxl.drawing.image import Image
from openpyxl.styles import Alignment, Font
from openpyxl.styles import borders
from openpyxl.styles.borders import Border
from openpyxl.worksheet import pagebreak
from openpyxl.utils import get_column_letter

class XLSREPORT:
    '''
    Excel book and functions \n
    DEBUG:\n
        - En el caso de WorkSheet en __init__ hay que preguntar y no usar try/except
        - Hay que depurar el tratamiento de archivos (repetidos, protegidos, etc)
    '''
    def __init__(self, filePath: str, FONT="Arial", WS_NAME="Data"):
        self.FILE = filePath
        self.FILE_STR = self.FILE + ".xlsx"
        self.fileName = os.path.basename(self.FILE_STR)
        self.ROW = 1
        self.FONT = FONT
        self.WS_NAME = WS_NAME
        ## WORKBOOK
        try:
            self.WB = xls.load_workbook(self.FILE_STR)
            # print("LOAD")
        except:
            self.WB = xls.Workbook(self.FILE_STR)
            self.WB.create_sheet(self.WS_NAME)
            self.WB.save(self.FILE_STR)
            self.WB.close
            self.WB = xls.load_workbook(self.FILE_STR)
        ## WORKSHEET
        try:
            self.WS = self.WB[self.WS_NAME]
        except:
            self.WB.create_sheet(self.WS_NAME)
            self.WS = self.WB[self.WS_NAME]
        # self.WS.dimensions.ColumnDimension(self.WS, bestFit=True)

    def SAVE(self):
        self.WB.save(self.FILE_STR)

    def CLOSE(self):
        self.WB.close

    def SHEET_NEW(self, sheet_name: str):
        '''
        Create a new excel sheet
        '''
        self.WB.create_sheet(sheet_name)

    def SHEET_SELECT(self, sheet_name):
        self.WS = self.WB[sheet_name]

    def WR(self, ROW: int = 1, COLUMN: int = 1, VALUE="", f_size: int = 10, f_bolf=False, save: bool = False):
        # self.WS["A4"].value = VALUE
        self.WS.cell(ROW, COLUMN).value = VALUE
        self.WS.cell(ROW, COLUMN).alignment = Alignment(horizontal='left', vertical='center')
        self.WS.cell(ROW, COLUMN).font = Font(name=self.FONT, size=f_size, bold=f_bolf,)
        if save == True: self.SAVE()
    
    def WR_TITLE(self, ROW: int = 1, COLUMN: int = 1, VALUE="", save: bool = False):
        self.WS.cell(ROW, COLUMN).value = VALUE
        self.WS.cell(ROW, COLUMN).alignment = Alignment(horizontal='left', vertical='center')
        self.WS.cell(ROW, COLUMN).font = Font(name=self.FONT, size=12, bold=True,)
        self.ROW_WIDTH(ROW, 40)
        if save: self.SAVE()

    def WR_HEADER(self, ROW: int = 1, COLUMN: int = 1, VALUE="", VERTICAL: str = "center", wrap: bool = False, save: bool = False):
        '''
        Write in cell with HEADER style font
        Bold, Size 10
        VERTICAL: "top", "center", "bottom", "justify", "distributed"
        '''
        self.WS.cell(ROW, COLUMN).value = VALUE
        self.WS.cell(ROW, COLUMN).alignment = Alignment(horizontal='left', vertical=VERTICAL, wrap_text=wrap)
        self.WS.cell(ROW, COLUMN).font = Font(name=self.FONT, size=10, bold=True)
        if save: 
            self.SAVE()

    def WR_HEADERS(self, ROW: int = 1, HEADERS: list = list, VERTICAL: str = "center", wrap: bool = False, save: bool = False):
        '''
        Write and edit format of Headers List
        '''
        for head in HEADERS:
            self.WR_HEADER(ROW=ROW, COLUMN=HEADERS.index(head)+1, VALUE=head, VERTICAL=VERTICAL, wrap=wrap)
        self.ROW_WIDTH(ROW, 35)
        if save:
            self.SAVE()

    def ROW_INC(self, NUMBER=1):
        '''
        Add an increment in row count
        '''
        self.ROW = self.ROW + int(NUMBER)

    def ROW_GET(self):
        '''
        Get current row
        '''
        FILA = self.ROW
        return int(FILA)

    def ROW_WIDTH(self, ROW=1, WIDTH=10, save: bool = False):
        '''
        Set height of a row
        '''
        self.WS.row_dimensions[ROW].height = WIDTH
        # 
        if save == True: self.SAVE()

    def COL_WIDTH(self, COL: int = 1, WIDTH: int = 20, save=False):
        '''
        '''
        COL_LETT = COLUMN_STR(COL)
        self.WS.column_dimensions[COL_LETT].width = WIDTH
        if save == True: self.SAVE()

    def COL_AUTOFIT(self, save=False):
        '''
        '''
        for column_cells in self.WS.columns:
                new_column_length = max(len(str(cell.value)) for cell in column_cells)
                new_column_letter = (xls.utils.get_column_letter(column_cells[0].column))
                if new_column_length > 0:
                    self.WS.column_dimensions[new_column_letter].width = new_column_length*1.23
        if save == True: self.SAVE()
    
    def COL_FILTERS(self, save: bool =False):
        '''
        Set filters in current WorkSheet from A1 to maximun column and maximun row
        '''
        fullRange = f"A1:{get_column_letter(self.WS.max_column)}{self.WS.max_row}"
        self.WS.auto_filter.ref = fullRange
        if save == True: self.SAVE()

    def LOW_BORDER(self, ROW=1, col_ini=1, col_fin=300, save=False):
        '''
        INCOMPLETE, DEBUG:
        Hay que saber bien el diseño y todas las funciones de borders
        https://openpyxl.readthedocs.io/en/stable/styles.html?highlight=border_style
        '''
        # Style = "thick" (Grueso)
        border0 = borders.Side(style = None, color = None, border_style = None)
        borderLow = borders.Side(
            style = "medium", 
            color="000000", 
            # border_style = "double"
            )
        thin = Border(left = border0, right = border0, bottom = borderLow, top = border0)
        for col in range(col_ini, col_fin): self.WS.cell(row=ROW, column=col).border = thin
        if save == True: self.SAVE()

    # def MERGE(self, ROW_INI, COL_INI, ROW_FIN, COL_FIN, save=False):
    #     self.WS.merge_cells(
    #         start_row = ROW_INI, 
    #         start_column = COL_INI, 
    #         end_row = ROW_FIN, 
    #         end_column = COL_FIN)
    #     if save == True: self.SAVE()

    def PRNT_AREA(self, COL_FIN: int, save: bool = False):
        '''
        Ajusta la zona de impresion
        INCOMPLETE
        '''
        # self.WS = self.WB[SHEET]
        # self.WS.page_setup.orientation = self.WS.ORIENTATION_LANDSCAPE
        self.WS.page_setup.fitToPage = True
        self.WS.page_setup.fitToPage = 1
        self.WS.page_setup.fitToHeight = False
        COL_STR = COLUMN_STR(COL_FIN)
        self.WS.print_area = "A:" + COL_STR
        if save == True: self.SAVE()

    def SHEET_HEAD(self, ROW_FIN: int, save: bool = False):
        '''
        '''
        self.WS.print_title_rows = "1:" + str(ROW_FIN)
        self.WS.page_margins.top = 0.4
        self.WS.page_margins.botom = 0.4
        # self.WS.page_margins.header = 0.7
        self.WS.page_margins.header = 0.4
        self.WS.page_margins.footer = 0.4
        if save == True: self.SAVE()

    def PAGE_BREAK(self, ROW: int = 1, save: bool = False):
        '''
        Insert a page break in selected row
        '''
        page_break = pagebreak.Break(id=ROW-1)
        break_list = self.WS.row_breaks
        break_list.append(page_break)
        self.WS.row_breaks = break_list
        if save == True: self.SAVE()

    def IMAGE_INSERT(self, ROW: int = 1, COLUMN: int = 1, HEIGHT=None, WIDTH=None):
        '''
        * Necesary install Pillow packages
        INCOMPLETE
        Sin usar ni comprobar
        '''
        PATH = "C:/Users/GONZA_PA/Desktop/rus-logo.png"
        img = Image(PATH)
        ## PIXEL VALUE
        if HEIGHT and WIDTH: 
            img.height = HEIGHT
            img.width = WIDTH
        cell_str = xls.utils.get_column_letter(COLUMN) + str(ROW)
        self.WS.add_image(img, cell_str)

def CELL_STR(ROW: int, COLUMN: int) -> str:
    '''
    '''
    col_str = xls.utils.get_column_letter(COLUMN)
    cellstr = "{}{}".format(col_str, ROW)
    return cellstr

def COLUMN_STR(COLUMN: int) -> str:
    '''
    '''
    letter = xls.utils.get_column_letter(COLUMN)
    return letter

def DF_REPORT(DATAFRAME: pd.DataFrame, fileName: str, fontName: str = 'Calibri') -> None:
    '''
    Create excel report from selected Pandas DataFrame
    '''
    report = XLSREPORT(fileName, fontName)
    ## HEADERS
    HEADERS: list = DATAFRAME.columns.values.tolist()
    report.WR_HEADERS(1, HEADERS, VERTICAL="top")
    report.COL_FILTERS()
    report.LOW_BORDER(report.ROW, col_fin=len(HEADERS)+1)
    report.ROW_INC()
    ## DATA
    for row in range(len(DATAFRAME.index)):
        row_data = list(DATAFRAME.iloc[row].values)
        for value in row_data:
            report.WR(report.ROW, row_data.index(value)+1, value)
        report.ROW_INC()
    report.COL_AUTOFIT()
    ## FIN
    report.SAVE()


''' 
--------------------------------------------------------
'''